@@ -13,13 +13,17 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import mock
+from unittest import mock
 
-from neutron import context as n_ctx
-from neutron.plugins.common import constants as nconstants
+from neutron.db import servicetype_db as st_db
+from neutron_lib import context as n_ctx
+from neutron_lib.plugins import constants
+from neutron_lib.plugins import directory
 from oslo_utils import uuidutils
 
+from neutron_vpnaas.services.vpn import plugin as vpn_plugin
 from neutron_vpnaas.services.vpn.service_drivers import ipsec as ipsec_driver
+from neutron_vpnaas.services.vpn.service_drivers import ipsec_validator
 from neutron_vpnaas.tests import base
 
 
@@ -48,22 +52,49 @@ class FakeSqlQueryObject(dict):
         super(FakeSqlQueryObject, self).__init__(**entries)
 
 
+class TestValidatorSelection(base.BaseTestCase):
+
+    def setUp(self):
+        super(TestValidatorSelection, self).setUp()
+        vpnaas_provider = [{
+            'service_type': constants.VPN,
+            'name': 'vpnaas',
+            'driver': IPSEC_SERVICE_DRIVER,
+            'default': True
+        }]
+        # override the default service provider
+        self.service_providers = (
+            mock.patch.object(st_db.ServiceTypeManager,
+                              'get_service_providers').start())
+        self.service_providers.return_value = vpnaas_provider
+        mock.patch('neutron_lib.rpc.Connection').start()
+        stm = st_db.ServiceTypeManager()
+        stm.get_provider_names_by_resource_ids = mock.Mock(
+            return_value={})
+        mock.patch('neutron.db.servicetype_db.ServiceTypeManager.get_instance',
+                   return_value=stm).start()
+        mock.patch('neutron_vpnaas.db.vpn.vpn_db.VPNPluginDb.get_vpnservices',
+                   return_value=[]).start()
+        self.vpn_plugin = vpn_plugin.VPNDriverPlugin()
+
+    def test_reference_driver_used(self):
+        default_provider = self.vpn_plugin.default_provider
+        default_driver = self.vpn_plugin.drivers[default_provider]
+        self.assertIsInstance(default_driver.validator,
+                              ipsec_validator.IpsecVpnValidator)
+
+
 class TestIPsecDriver(base.BaseTestCase):
     def setUp(self):
         super(TestIPsecDriver, self).setUp()
-        mock.patch('neutron.common.rpc.create_connection').start()
+        mock.patch('neutron_lib.rpc.Connection').start()
 
         l3_agent = mock.Mock()
         l3_agent.host = FAKE_HOST
         plugin = mock.Mock()
         plugin.get_l3_agents_hosting_routers.return_value = [l3_agent]
-        plugin_p = mock.patch('neutron.manager.NeutronManager.get_plugin')
-        get_plugin = plugin_p.start()
-        get_plugin.return_value = plugin
-        service_plugin_p = mock.patch(
-            'neutron.manager.NeutronManager.get_service_plugins')
-        get_service_plugin = service_plugin_p.start()
-        get_service_plugin.return_value = {nconstants.L3_ROUTER_NAT: plugin}
+        directory.add_plugin(constants.CORE, plugin)
+        directory.add_plugin(constants.L3, plugin)
         self.svc_plugin = mock.Mock()
         self.svc_plugin.get_l3_agents_hosting_routers.return_value = [l3_agent]
         self._fake_vpn_router_id = _uuid()
@@ -71,6 +102,8 @@ class TestIPsecDriver(base.BaseTestCase):
             'router_id': self._fake_vpn_router_id
         }
         self.driver = ipsec_driver.IPsecVPNDriver(self.svc_plugin)
+        self.validator = ipsec_validator.IpsecVpnValidator(self.driver)
+        self.context = n_ctx.get_admin_context()
 
     def _test_update(self, func, args, additional_info=None):
         ctxt = n_ctx.Context('', 'somebody')
@@ -170,7 +203,8 @@ class TestIPsecDriver(base.BaseTestCase):
                                              external_v4_ip=external_ip,
                                              external_v6_ip=None,
                                              subnet_id=subnet_id,
-                                             router_id='foo-router-id')
+                                             router_id='foo-router-id',
+                                             project_id='foo-project-id')
         if local_endpoints:
             fake_vpnservice.subnet = None
         else:
@@ -211,6 +245,8 @@ class TestIPsecDriver(base.BaseTestCase):
                            'network_id': 'foo-net-id'},
                 'subnet_id': 'foo-subnet-id',
                 'external_ip': external_ip,
+                'project_id': 'foo-project-id',
+                'tenant_id': 'foo-project-id',
                 'ipsec_site_connections': [
                     {'id': 'conn-id',
                      'peer_id': peer_id,
@@ -250,6 +286,8 @@ class TestIPsecDriver(base.BaseTestCase):
                 'subnet': None,
                 'subnet_id': None,
                 'external_ip': external_ip,
+                'project_id': 'foo-project-id',
+                'tenant_id': 'foo-project-id',
                 'ipsec_site_connections': [
                     {'id': 'conn-id',
                      'peer_id': peer_id,
@@ -419,3 +457,10 @@ class TestIPsecDriver(base.BaseTestCase):
         self.driver.create_vpnservice(ctxt, vpnservice_dict)
         self.svc_plugin.set_external_tunnel_ips.assert_called_once_with(
             ctxt, FAKE_SERVICE_ID, v4_ip='10.0.0.99', v6_ip='2001::1')
+
+    def test_validate_ipsec_policy(self):
+        # Validate IPsec Policy transform_protocol
+        ipsec_policy = {'transform_protocol': 'ah-esp'}
+        self.assertRaises(ipsec_validator.IpsecValidationFailure,
+                          self.validator.validate_ipsec_policy,
+                          self.context, ipsec_policy)
